@@ -248,6 +248,7 @@
 #include "llvm/Support/Path.h"
 #include <algorithm>
 #include <iterator>
+#include <map>
 #include <string>
 #include <vector>
 
@@ -480,7 +481,7 @@ struct HeaderEntry {
 
 typedef std::vector<HeaderEntry> HeaderContents;
 
-class EntityMap : public StringMap<SmallVector<Entry, 2> > {
+class EntityMap : public std::map<std::string, SmallVector<Entry, 2>> {
 public:
   DenseMap<const FileEntry *, HeaderContents> HeaderContentMismatches;
 
@@ -561,10 +562,7 @@ public:
   bool TraverseTemplateArgumentLoc(const TemplateArgumentLoc &ArgLoc) {
     return true;
   }
-  bool TraverseTemplateArguments(const TemplateArgument *Args,
-                                 unsigned NumArgs) {
-    return true;
-  }
+  bool TraverseTemplateArguments(ArrayRef<TemplateArgument>) { return true; }
   bool TraverseConstructorInitializer(CXXCtorInitializer *Init) { return true; }
   bool TraverseLambdaCapture(LambdaExpr *LE, const LambdaCapture *C,
                              Expr *Init) {
@@ -750,10 +748,7 @@ public:
   bool TraverseTemplateArgumentLoc(const TemplateArgumentLoc &ArgLoc) {
     return true;
   }
-  bool TraverseTemplateArguments(const TemplateArgument *Args,
-    unsigned NumArgs) {
-    return true;
-  }
+  bool TraverseTemplateArguments(ArrayRef<TemplateArgument>) { return true; }
   bool TraverseConstructorInitializer(CXXCtorInitializer *Init) { return true; }
   bool TraverseLambdaCapture(LambdaExpr *LE, const LambdaCapture *C,
                              Expr *Init) {
@@ -941,7 +936,7 @@ int main(int Argc, const char **Argv) {
         continue;
       LocationArray::iterator FI = DI->begin();
       StringRef kindName = Entry::getKindName((Entry::EntryKind)KindIndex);
-      errs() << "error: " << kindName << " '" << E->first()
+      errs() << "error: " << kindName << " '" << E->first
              << "' defined at multiple locations:\n";
       for (LocationArray::iterator FE = DI->end(); FI != FE; ++FI) {
         errs() << "    " << FI->File->getName() << ":" << FI->Line << ":"
